@@ -1,5 +1,7 @@
 """Implements nose test program and collector.
 """
+from __future__ import generators
+
 import logging
 import os
 import re
@@ -14,8 +16,8 @@ from nose.config import Config
 from nose.loader import defaultTestLoader
 from nose.proxy import ResultProxySuite
 from nose.result import Result
-from nose.suite import LazySuite
-from nose.util import absdir
+from nose.suite import LazySuite, TestModule
+from nose.util import absdir, tolist
 from nose.importer import add_path
 
 log = logging.getLogger('nose.core')
@@ -37,8 +39,9 @@ class TestCollector(LazySuite):
         self.path = conf.where
         
     def loadtests(self):
-        for test in self.loader.loadTestsFromDir(self.path):
-            yield test
+        for path in tolist(self.path):
+            for test in self.loader.loadTestsFromDir(path):
+                yield test
             
     def __repr__(self):
         return "collector in %s" % self.path
@@ -62,7 +65,8 @@ def collector():
     setuptools_incompat = ( 'finalize', 'prepareTest', 'report',
                             'setOutputStream')
     
-    conf = configure(env=os.environ, disable_plugins=setuptools_incompat)
+    conf = configure(argv=[], env=os.environ,
+                     disable_plugins=setuptools_incompat)
     Result.conf = conf
     loader = defaultTestLoader(conf)
     loader.suiteClass = ResultProxySuite
@@ -103,7 +107,7 @@ class TextTestRunner(unittest.TextTestRunner):
 
     
 class TestProgram(unittest.TestProgram):
-    """usage: %prog [options] [names]
+    r"""usage: %prog [options] [names]
     
     nose provides an alternate test discovery and running process for
     unittest, one that is intended to mimic the behavior of py.test as much as
@@ -112,8 +116,8 @@ class TestProgram(unittest.TestProgram):
     nose collects tests automatically from python source files,
     directories and packages found in its working directory (which
     defaults to the current working directory). Any python source file,
-    directory or package that matches the testMatch regulat expression
-    (by default: (?:^|[\\b_\\.-])[Tt]est) will be collected as a test (or
+    directory or package that matches the testMatch regular expression
+    (by default: (?:^|[\b_\.-])[Tt]est) will be collected as a test (or
     source for collection of tests). In addition, all other packages
     found in the working directory are examined for python source files
     or directories that match testMatch. Package discovery descends all
@@ -229,25 +233,21 @@ class TestProgram(unittest.TestProgram):
         result = self.testRunner.run(self.test)
         self.success = result.wasSuccessful()
         return self.success
-        
-def configure(argv=None, env=None, help=False, disable_plugins=None):
-    """Configure the nose running environment. Execute configure before
-    collecting tests with nose.TestCollector to enable output capture and
-    other features.
-    """
-    if argv is None:
-        argv = sys.argv
-    if env is None:
-        env = os.environ
-    
-    conf = Config()
-    parser = OptionParser(TestProgram.__doc__)
+
+def get_parser(env=None, builtin_only=False, doc=None):
+    if doc is None:
+        doc = TestProgram.__doc__
+    parser = OptionParser(doc)
     parser.add_option("-V","--version",action="store_true",
                       dest="version",default=False,
                       help="Output nose version and exit")
-    parser.add_option("-v", "--verbose", action="count",
-                      dest="verbosity", default=int(env.get('NOSE_VERBOSE', 1)),
+    parser.add_option("-v", "--verbose",
+                      action="count", dest="verbosity",
+                      default=int(env.get('NOSE_VERBOSE', 1)),
                       help="Be more verbose. [NOSE_VERBOSE]")
+    parser.add_option("--verbosity", action="store", dest="verbosity",
+                      type="int", help="Set verbosity; --verbosity=2 is "
+                      "the same as -vv")
     parser.add_option("-l", "--debug", action="store",
                       dest="debug", default=env.get('NOSE_DEBUG'),
                       help="Activate debug logging for one or more systems. "
@@ -260,17 +260,18 @@ def configure(argv=None, env=None, help=False, disable_plugins=None):
                       "(default: sys.stderr)")
     parser.add_option("-q", "--quiet", action="store_const",
                       const=0, dest="verbosity")
-    parser.add_option("-w", "--where", action="store", dest="where",
-                      default=env.get('NOSE_WHERE', os.getcwd()),
+    parser.add_option("-w", "--where", action="append", dest="where",
                       help="Look for tests in this directory [NOSE_WHERE]")
-    parser.add_option("-e", "--exclude", action="store", dest="exclude",
-                      default=env.get('NOSE_EXCLUDE'),
+    parser.add_option("-e", "--exclude", action="append", dest="exclude",
                       help="Don't run tests that match regular "
                       "expression [NOSE_EXCLUDE]")
-    parser.add_option("-i", "--include", action="store", dest="include",
-                      default=env.get('NOSE_INCLUDE'),
+    parser.add_option("-i", "--include", action="append", dest="include",
                       help="Also run tests that match regular "
                       "expression [NOSE_INCLUDE]")
+    parser.add_option("-m", "--match", "--testmatch", action="store",
+                      dest="test_match", help="Use this regular expression to "
+                      "find tests [NOSE_TESTMATCH]",
+                      default=env.get('NOSE_TESTMATCH'))
     parser.add_option("-s", "--nocapture", action="store_false",
                       default=not env.get('NOSE_NOCAPTURE'), dest="capture",
                       help="Don't capture stdout (any stdout output "
@@ -311,13 +312,27 @@ def configure(argv=None, env=None, help=False, disable_plugins=None):
     # add opts from plugins
     all_plugins = []
     # when generating the help message, load only builtin plugins
-    for plugcls in load_plugins(others=not help):
+    for plugcls in load_plugins(others=not builtin_only):
         plug = plugcls()
         try:
             plug.add_options(parser, env)
         except AttributeError:
             pass
-        all_plugins.append(plug)
+    
+    return parser
+
+def configure(argv=None, env=None, help=False, disable_plugins=None):
+    """Configure the nose running environment. Execute configure before
+    collecting tests with nose.TestCollector to enable output capture and
+    other features.
+    """
+    if argv is None:
+        argv = sys.argv
+    if env is None:
+        env = os.environ
+    
+    conf = Config()
+    parser = get_parser(env=env, builtin_only=help)
         
     options, args = parser.parse_args(argv)
     if help:
@@ -334,9 +349,21 @@ def configure(argv=None, env=None, help=False, disable_plugins=None):
         print "%s version %s" % (os.path.basename(sys.argv[0]), __version__)
         sys.exit(0)
 
-    configure_logging(options)
+    # where is an append action, so it can't have a default value 
+    # in the parser, or that default will always be in the list
+    if not options.where:
+        options.where = env.get('NOSE_WHERE', os.getcwd())
+
+    # include and exclude also
+    if not options.include:
+        options.include = env.get('NOSE_INCLUDE', [])
+    if not options.exclude:
+        options.exclude = env.get('NOSE_EXCLUDE', [])
         
+    configure_logging(options)
+
     # hand options to plugins
+    all_plugins = [plug() for plug in load_plugins()]
     for plug in all_plugins:
         plug.configure(options, conf)
         if plug.enabled and disable_plugins:
@@ -345,11 +372,6 @@ def configure(argv=None, env=None, help=False, disable_plugins=None):
                     plug.enabled = False
                     log.warning("Plugin %s disabled: not all methods "
                                 "supported in this environment" % plug.name)
-                                
-        
-    # configuration works by setting class-level defaults
-    # in Test and properties in runner and loader
-
     conf.addPaths = options.addPaths
     conf.capture = options.capture
     conf.detailedErrors = options.detailedErrors
@@ -361,26 +383,36 @@ def configure(argv=None, env=None, help=False, disable_plugins=None):
     conf.includeExe = options.includeExe
     
     if options.where is not None:
-        conf.where = absdir(options.where)
-        if conf.where is None:
-            raise Exception("Working directory %s not found, or "
-                            "not a directory" % options.where)
-        log.info("Looking for tests in %s", conf.where)
-        if conf.addPaths and \
-            os.path.exists(os.path.join(conf.where, '__init__.py')):
-            log.info("Working directory is a package; adding to sys.path")
-            add_path(conf.where)
-        
+        conf.where = []
+        for path in tolist(options.where):
+            log.debug('Adding %s as nose working directory', path)
+            abs_path = absdir(path)
+            if abs_path is None:
+                raise ValueError("Working directory %s not found, or "
+                                 "not a directory" % path)
+            conf.where.append(abs_path)
+            log.info("Looking for tests in %s", abs_path)
+            if conf.addPaths and \
+                    os.path.exists(os.path.join(abs_path, '__init__.py')):
+                log.info("Working directory %s is inside of a package; "
+                         "adding package root to sys.path" % abs_path)
+                add_path(abs_path)
+                
     if options.include:
-        conf.include = re.compile(options.include)
+        conf.include = map(re.compile, tolist(options.include))
         log.info("Including tests matching %s", options.include)
+        
     if options.exclude:
-        conf.exclude = re.compile(options.exclude)
+        conf.exclude = map(re.compile, tolist(options.exclude))
         log.info("Excluding tests matching %s", options.exclude)
+
+    if options.test_match:
+        conf.testMatch = re.compile(options.test_match)
+        log.info("Test match regular expression: %s", options.test_match)
         
     if conf.capture:
         start_capture()
-
+        
     try:
         # give plugins a chance to start
         call_plugins(conf.plugins, 'begin')
@@ -414,7 +446,7 @@ def configure_logging(options):
     # default level    
     lvl = logging.WARNING
     if options.verbosity >= 5:
-        lvl = 0
+        lvl = 1
     elif options.verbosity >= 4:
         lvl = logging.DEBUG
     elif options.verbosity >= 3:
@@ -446,6 +478,16 @@ def run(*arg, **kw):
     result = TestProgram(*arg, **kw).success
     end_capture()
     return result
-            
+
+def runmodule(name='__main__'):
+    """Collect and run tests in a single module only. Defaults to running
+    tests in __main__.
+    """
+    conf = configure()
+    testLoader = defaultTestLoader(conf)
+    def collector(conf, loader):
+        return TestModule(loader.loadTestsFromModule, conf, name)
+    main(defaultTest=collector, testLoader=testLoader)    
+
 if __name__ == '__main__':
     main()
