@@ -14,6 +14,7 @@ import logging
 import os
 import sys
 from nose.plugins.base import Plugin
+from nose.util import tolist
 
 log =  logging.getLogger(__name__)
 
@@ -48,6 +49,15 @@ class Coverage(Plugin):
                           default=env.get('NOSE_COVER_TESTS'),
                           help="Include test modules in coverage report "
                           "[NOSE_COVER_TESTS]")
+        parser.add_option("--cover-inclusive", action="store_true",
+                          dest="cover_inclusive",
+                          default=env.get('NOSE_COVER_INCLUSIVE'),
+                          help="Include all python files under working "
+                          "directory in coverage report.  Useful for "
+                          "discovering holes in test coverage if not all "
+                          "files are imported by the test suite. "
+                          "[NOSE_COVER_INCLUSIVE]")
+        
 
     def configure(self, options, config):
         Plugin.configure(self, options, config)
@@ -62,7 +72,8 @@ class Coverage(Plugin):
         self.conf = config
         self.coverErase = options.cover_erase
         self.coverTests = options.cover_tests
-        self.coverPackages = self.tolist(options.cover_packages)
+        self.coverPackages = tolist(options.cover_packages)
+        self.coverInclusive = options.cover_inclusive
         if self.coverPackages:
             log.info("Coverage report will include only packages: %s",
                      self.coverPackages)
@@ -74,12 +85,15 @@ class Coverage(Plugin):
         if self.coverErase:
             log.debug("Clearing previously collected coverage statistics")
             coverage.erase()
+        coverage.exclude('#pragma[: ]+[nN][oO] [cC][oO][vV][eE][rR]')
         coverage.start()
         
     def report(self, stream):
         log.debug("Coverage report")
         import coverage
         coverage.stop()
+        coverage.the_coverage.save()
+        
         modules = [ module
                     for name, module in sys.modules.items()
                     if self.wantModuleCoverage(name, module) ]
@@ -112,3 +126,17 @@ class Coverage(Plugin):
         # coverPackages is on -- in that case, if we wanted this
         # module, we would have already returned True
         return not self.coverPackages
+    
+    def wantFile(self, file, package=None):    
+        """If inclusive coverage enabled, return true for all source files 
+        in wanted packages."""    
+        if self.coverInclusive:
+            if file.endswith(".py"): 
+                if package and self.coverPackages:
+                    for want in self.coverPackages:
+                        if package.startswith(want):
+                            return True
+                else:
+                    return True
+        return None
+    
