@@ -2,6 +2,8 @@
 classes and directories, and provide suite-level fixtures (setUp/tearDown
 methods).
 """
+from __future__ import generators
+
 import logging
 import sys
 import unittest
@@ -27,6 +29,9 @@ class LazySuite(unittest.TestSuite):
             conf = Config()
         self.conf = conf
 
+    def __iter__(self):
+        return iter(self._tests)
+        
     def loadtests(self):
         for test in self._loadtests():
             yield test
@@ -40,26 +45,30 @@ class LazySuite(unittest.TestSuite):
         self.run(*arg, **kw)
         
     def run(self, result):
+        result.startTest(self)
         try:
-            self.setUp()
-        except KeyboardInterrupt:
-            raise
-        except:
-            result.addError(self, sys.exc_info())
-            return
-        for test in self._tests:
-            log.debug("running test %s", test)
-            if result.shouldStop:
-                break
-            test(result)
-        try:
-            self.tearDown()
-        except KeyboardInterrupt:
-            raise
-        except:
-            result.addError(self, sys.exc_info())            
-        return result
-
+            try:
+                self.setUp()
+            except KeyboardInterrupt:
+                raise
+            except:
+                result.addError(self, sys.exc_info())
+                return
+            for test in self._tests:
+                log.debug("running test %s", test)
+                if result.shouldStop:
+                    break
+                test(result)
+            try:
+                self.tearDown()
+            except KeyboardInterrupt:
+                raise
+            except:
+                result.addError(self, sys.exc_info())            
+            return result
+        finally:
+            result.stopTest(self)
+            
     def setUp(self):
         pass
         
@@ -87,11 +96,14 @@ class GeneratorMethodTestSuite(LazySuite):
             except ValueError:
                 test_method, arg = test[0], tuple()
             log.debug('test_method: %s, arg: %s', test_method, arg)
+            case = None
             if callable(test_method):
                 name = test_method.__name__
+                case = test_method
             else:
                 name = test_method
-            yield MethodTestCase(self.cls, name, self.method, *arg)
+            yield MethodTestCase(
+                self.cls, name, self.method, case, *arg)
 
             
 class TestClass(LazySuite):
@@ -183,9 +195,11 @@ class TestModule(LazySuite):
         function may optionally accept a single argument; in that case,
         the test package or module will be passed to the setup function.
         """
+        log.debug('TestModule.setUp')
         if self.module is None:
             self.module = _import(self.moduleName, [self.path], self.conf)
-
+            log.debug('Imported %s from %s on %s', self.module,
+                      self.moduleName, self.path)
         if hasattr(self.module, '__path__'):
             names = ['setupPackage', 'setUpPackage', 'setup_package']
         else:
