@@ -2,6 +2,7 @@
 
 from docutils.core import publish_string, publish_parts
 import nose
+import nose.commands
 import nose.tools
 import os
 import re
@@ -19,9 +20,13 @@ docs = publish_parts(txt, writer_name='html')
 docs.update({'version': nose.__version__,
              'date': time.ctime()})
 
-# print "Tools..."
-# tools = publish_parts(nose.tools.__doc__, writer_name='html')
-# docs['tools'] = tools['body']
+#print "Tools..."
+#tools = publish_parts(nose.tools.__doc__, writer_name='html')
+#docs['tools'] = tools['body']
+
+print "Commands..."
+cmds = publish_parts(nose.commands.__doc__, writer_name='html')
+docs['commands'] = cmds['body']
 
 print "Changelog..."
 changes = open(os.path.join(root, 'CHANGELOG'), 'r').read()
