@@ -1,5 +1,7 @@
+import re
 import unittest
 import nose.config
+from nose.core import configure
 
 class TestNoseConfig(unittest.TestCase):
 
@@ -22,5 +24,13 @@ class TestNoseConfig(unittest.TestCase):
         c.update({'exclude':'x'})
         assert c.exclude == 'x'
 
+    def test_multiple_include(self):
+        conf = configure(['--include=a', '--include=b'])
+        self.assertEqual(conf.include, [re.compile('a'), re.compile('b')])
+
+    def test_single_include(self):
+        conf = configure(['--include=b'])
+        self.assertEqual(conf.include, [re.compile('b')])
+
 if __name__ == '__main__':
     unittest.main()
