@@ -5,7 +5,7 @@ import unittest
 import nose.plugins
 from optparse import OptionParser
 import tempfile
-from warnings import warn
+from warnings import warn, filterwarnings, resetwarnings
 
 from nose.config import Config
 from nose.plugins.attrib import AttributeSelector
@@ -14,6 +14,7 @@ from nose.plugins.cover import Coverage
 from nose.plugins.doctests import Doctest
 from nose.plugins.missed import MissedTests
 from nose.plugins.prof import Profile
+from nose.plugins.isolate import IsolationPlugin
 
 from mock import *
 
@@ -31,8 +32,8 @@ class ErrPkgResources(object):
 
         
 # some plugins have 2.4-only features
-pyvers = sys.version_info
-compat_24 = pyvers[0] >= 2 and pyvers[1] >= 4
+compat_24 = sys.version_info >= (2, 4)
+
 
 class TestBuiltinPlugins(unittest.TestCase):
 
@@ -51,7 +52,8 @@ class TestBuiltinPlugins(unittest.TestCase):
         assert AttributeSelector in plugs
         assert Profile in plugs
         assert MissedTests in plugs
-        assert len(plugs) == 5
+        assert IsolationPlugin in plugs
+        assert len(plugs) == 6
 
         for p in plugs:
             assert not p.enabled
@@ -60,14 +62,13 @@ class TestBuiltinPlugins(unittest.TestCase):
         tmp = nose.plugins.pkg_resources
         nose.plugins.pkg_resources = ErrPkgResources()
         try:
-            # turn off logging
-            lvl = logging.getLogger('nose.plugins').level
-            logging.getLogger('nose.plugins').setLevel(100)            
+            # turn off warnings
+            filterwarnings('ignore', category=RuntimeWarning)
             plugs = list(nose.plugins.load_plugins(builtin=True, others=True))
             self.assertEqual(plugs, [])
         finally:
             nose.plugins.pkg_resources = tmp
-            logging.getLogger('nose.plugins').setLevel(lvl)
+            resetwarnings()
             
     def test_add_options(self):
         conf = Config()
@@ -88,12 +89,6 @@ class TestBuiltinPlugins(unittest.TestCase):
         plug.configure(opt, conf)
         assert plug.enabled
 
-    def test_tolist(self):
-        plug = P()
-        assert plug.tolist('foo') == ['foo']
-        assert plug.tolist(['foo', 'bar']) == ['foo', 'bar']
-        assert plug.tolist('foo,bar') == ['foo', 'bar']
-        self.assertEqual(plug.tolist('.*foo/.*,.1'), ['.*foo/.*', '.1'])
         
 class TestDoctestPlugin(unittest.TestCase):
 
@@ -122,7 +117,31 @@ class TestDoctestPlugin(unittest.TestCase):
             assert o3[0] == '--doctest-extension'
         else:
             assert len(parser.opts) == 2
-    
+
+    def test_config(self):
+        if not compat_24:
+            return
+        # test that configuration works properly when both environment
+        # and command line specify a doctest extension
+        parser = OptionParser()
+        env = {'NOSE_DOCTEST_EXTENSION':'ext'}
+        argv = ['--doctest-extension', 'txt']
+        dtp = Doctest()
+        dtp.add_options(parser, env)
+        options, args = parser.parse_args(argv)
+        
+        print options
+        print args
+        self.assertEqual(options.doctestExtension, ['ext', 'txt'])
+
+        env = {}
+        parser = OptionParser()
+        dtp.add_options(parser, env)
+        options, args = parser.parse_args(argv)
+        print options
+        print args
+        self.assertEqual(options.doctestExtension, ['txt'])
+            
     def test_want_file(self):
         # doctest plugin can select module and/or non-module files
         conf = Config()
@@ -257,6 +276,20 @@ class TestAttribPlugin(unittest.TestCase):
         assert plug.wantFunction(f) is not False
         assert not plug.wantFunction(g)
 
+    def test_class_attr(self):
+        class TestP:
+            foo = True
+            def h():
+                pass
+
+        def i():
+            pass
+        
+        plug = AttributeSelector()
+        plug.attribs = [[('foo', True)]]
+        assert plug.wantMethod(TestP.h) is not False
+        assert plug.wantFunction(i) is False
+        
     def test_eval_attr(self):
         if not compat_24:
             warn("No support for eval attributes in python versions older"
