@@ -87,5 +87,42 @@ class TestUtils(unittest.TestCase):
                          (me, __name__, 'baz'))
         self.assertEqual(nose.test_address(foo_mtc),
                          (me, __name__, 'Foo.bar'))
+
+    def test_tolist(self):
+        from nose.util import tolist
+        assert tolist('foo') == ['foo']
+        assert tolist(['foo', 'bar']) == ['foo', 'bar']
+        assert tolist('foo,bar') == ['foo', 'bar']
+        self.assertEqual(tolist('.*foo/.*,.1'), ['.*foo/.*', '.1'])
+
+    def test_try_run(self):
+        from nose.util import try_run
+        import imp
+
+        def bar():
+            pass
+
+        def bar_m(mod):
+            pass
+
+        class Bar:
+            def __call__(self):
+                pass
+
+        class Bar_m:
+            def __call__(self, mod):
+                pass
+        
+        foo = imp.new_module('foo')
+        foo.bar = bar
+        foo.bar_m = bar_m
+        foo.i_bar = Bar()
+        foo.i_bar_m = Bar_m()
+
+        try_run(foo, ('bar',))
+        try_run(foo, ('bar_m',))
+        try_run(foo, ('i_bar',))
+        try_run(foo, ('i_bar_m',))
+        
 if __name__ == '__main__':
     unittest.main()
